/*  Copyright (c) 2005, Applied Security, Inc.
    Jared DeMott, jdemott@appliedsec.com, www.appliedsec.com

    This file is part of cmdline.

    Cmdline is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CmdLine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Cmdline; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include "userFunc.h"
#define MAX_PLACES 20
#define MAX 500
#define BUFFERMAX 1000000

void PRINT(int, const char *, const char *);
void Usage();
struct field
{
	char *Arg;		/* This is the ptr to the begining of the field  */
	int IsRequired; 	/* {default=0} Indicates if this field is required or not */
	int IsStatic;		/* {default=0} Indicates if this field is a static string */
	int AttackType[2];	/* Each field is 1 or 2*/
	int AttackCount;	/* {default=0} Is the number of types chosen (1-3). Max = 3. If = 0, then no attack is chosen. */
	int CycleType;		/* {default=0} 0=no cycle, 1=* cycle, 2=[] cycle  */
	int CycleCount;		/* {default=0} Count is 255 for * or variable for [...]  */
	char CycleData[255];	/* data is 0x01->0xff for * or variable for [...]  */
};

int main (int argc, char *argv[])
{
	FILE *fp;
	char ch;
	int BFchar;
	char *Binary, *Current, *Working, *Buffer;
	char line[MAX];
	int i, j, k, l, m, n, o, p, q, x, y, LineNum, ARGsCount, c, BFstart, BFstop, BFinc, NUMstart, NUMstop, NUMinc;
	struct field ARGs[MAX/2];
	char *Command, *CommandBeg;
	int OneRequiredField, OneNonRequiredField, OneCycleField, rMaxAttackCount, RMaxAttackCount;
	char *NUMarray[24] = {"-4294967295", "-4294967294", "-16777216", "-16777215", "-16777214", "-65536", "-65535", "-65534", "-256", "-255", "-254", "-1", "0", "254", "255", "256", "65534", "65535", "65536", "16777214", "16777215", "16777216", "4294967294", "4294967295"};
	int DefaultNumAttack = 1;
	int Loop, Seed, Time;
	float a;

	if(argc < 3)
	  {
	    Usage();
	    exit(1);
	  }

	if ( argc == 3)
	{
		if( (fp = fopen(argv[2], "r")) == NULL)
		{
			printf("\nError opening file!!\n\n");
			exit(-1);
		}
		BFstart=0;
		BFstop=18000;
		BFinc=100;
		BFchar=0x6e;

		NUMstart=1;
		NUMstop=10;
		NUMinc=1;
		Loop=6;
	
		Time=time();
		srand(Time);
		srand48(Time);
	}
c = getopt(argc, argv, "nbr");
  switch(c)
  {
    case 'b':
    {
	if ( argc == 7)
	{
		if( (fp = fopen(argv[2], "r")) == NULL)
		{
			printf("\nError opening file!!\n\n");
			exit(-1);
		}
		if( (atoi(argv[3]) || atoi(argv[4])) > BUFFERMAX)
		{
			printf("\nBuffer size to large!!\n");
			exit(-1);
		}
		else
		{
			BFstart=atoi(argv[3]);
			BFstop=atoi(argv[4]);
			BFinc=atoi(argv[5]);
			sscanf(argv[6], "%x", &BFchar);
		}
	}
	else if (argc != 3)
	{	
		Usage();
		exit(-1);
	}
	/*Read in the file and process it one line at a time*/
	memset(line, 0x00, MAX);
	LineNum=0;
	while (fgets (line, MAX, fp) != NULL)
	{
	  if(*line !=  '#')
	  {
		for(i=0; i<MAX/2; i++)
		{
			ARGs[i].Arg = NULL;		
		}	
		i=0;
		LineNum++;
		ARGsCount=0;
		Binary = strchr(line, 0x0a);
		*(Binary)=0x00;
		ARGs[i].Arg=line;
		printf("\n\tCmdline is parsing the following line: '%s'\n\n",ARGs[i].Arg);
		while ( (Binary = strchr(ARGs[i].Arg, 0x20)) != NULL )
		{
			*(Binary) = 0x00;
			i++;
			Binary++;
			ARGs[i].Arg=Binary;
			ARGsCount++;
		}
		ARGsCount++;
		for(i=0; i<ARGsCount; i++)
		{
			ARGs[i].IsRequired=0;
			ARGs[i].IsStatic=0;
			ARGs[i].AttackCount=0;
			ARGs[i].CycleType=0;
			ARGs[i].CycleCount=0;
		}
	
		/*This is the master loop that parses the ARGs on each line and builds the 'Command' buffer to call system on*/
		OneRequiredField = 0;
		OneNonRequiredField = 0;
		OneCycleField = 0;
		rMaxAttackCount = 0;
		RMaxAttackCount = 0;
		for(i=1; i<ARGsCount; i++)
		{
			/*First we look for the '_'*/
			Current=strchr(ARGs[i].Arg, '_');
			if (Current != NULL)
			{
				switch(*(Current+1))
				{
					case 'R':
						OneRequiredField = 1;
						ARGs[i].IsRequired=1;
						for(j=0; j<2; j++)
						{	
							switch(*(Current+2+j))
							{
								case '1':
									ARGs[i].AttackType[ARGs[i].AttackCount]=1;
									ARGs[i].AttackCount++;
									break;
								case '2':
									ARGs[i].AttackType[ARGs[i].AttackCount]=2;
									ARGs[i].AttackCount++;
									break;
								case 0x20:
									if(j==0)
									{
										printf("\nInvalid Indicator on line number %d\n", LineNum);
										printf("%s\n", ARGs[i].Arg);
										exit(-1);
									}
									j=3;
									break;
								case 0x00:
									if(j==0)
									{
										printf("\nInvalid Indicator on line number %d\n", LineNum);
										printf("%s\n", ARGs[i].Arg);
										exit(-1);
									}
									j=3;
									break;
								default:
									printf("\nInvalid Indicator on line number %d\n", LineNum);
									printf("%s\n", ARGs[i].Arg);
									exit(-1);
							}
						}
						if( RMaxAttackCount < ARGs[i].AttackCount )
						{
							RMaxAttackCount = ARGs[i].AttackCount;
						}
						break;
		
					case 'r':
						OneNonRequiredField = 1;
						ARGs[i].IsRequired=0;
						for(j=0; j<2; j++)
						{	
							switch(*(Current+2+j))
							{
								case '1':
									ARGs[i].AttackType[ARGs[i].AttackCount]=1;
									ARGs[i].AttackCount++;
									break;
								case '2':
									ARGs[i].AttackType[ARGs[i].AttackCount]=2;
									ARGs[i].AttackCount++;
									break;
								case 0x20:
									if(j==0)
									{
										printf("\nInvalid Indicator on line number %d\n", LineNum);
										printf("%s\n", ARGs[i].Arg);
										exit(-1);
									}
									j=3;
									break;
								case 0x00:
									if(j==0)
									{
										printf("\nInvalid Indicator on line number %d\n", LineNum);
										printf("%s\n", ARGs[i].Arg);
										exit(-1);
									}
									j=3;
									break;
								default:
									printf("\nInvalid Indicator on line number %d\n", LineNum);
									printf("%s\n", ARGs[i].Arg);
									exit(-1);
							}
						}
						if( rMaxAttackCount < ARGs[i].AttackCount )
						{
							rMaxAttackCount = ARGs[i].AttackCount;
						}
						break;
			
					default:
						ARGs[i].IsStatic=1;
						break;

				}
			}	
			if ( Current=strchr(ARGs[i].Arg, '*') )
			{
				OneCycleField = 1;
				ARGs[i].CycleType = 1;
				ARGs[i].CycleCount = 255;
				for (j=0; j<ARGs[i].CycleCount; j++)
				{
					ARGs[i].CycleData[j] = j+1;
				}
				*(Current) = 0x00;
			}
			else if ( Current=strchr(ARGs[i].Arg, '[') )
			{	
				OneCycleField = 1;
				ARGs[i].CycleType = 2;
				Working=strchr(ARGs[i].Arg, ']');
				ARGs[i].CycleCount = (Working - Current) - 1;
				for(j=0; j<ARGs[i].CycleCount; j++)
				{
					ARGs[i].CycleData[j] = *(Current + j + 1);
				}
				*(Current) = 0x00;
			}				
			else if ( (strchr(ARGs[i].Arg, '_')) == NULL) 
			{
				ARGs[i].IsStatic=1;
			}
		} /*Ends first PARSING loop through the fields on one line*/
		if ( (Command = malloc(ARGsCount*MAX_PLACES+MAX)) == NULL) 
		{
			printf("\n Memory error!!  Are there to many ARGs or too large a BUFFERMAX??\n\n");
			exit(-1);
		}
		CommandBeg=Command;
		Buffer=malloc(17);
		/*Array of field structures is populated. Begin Fuzzing*/
		if(OneRequiredField)
		{
		  for(l=0; l<RMaxAttackCount; l++)
		  {
			for(x=BFstart; x<=BFstop; x=x+BFinc)
			{
				memset(CommandBeg, 0x00, ARGsCount*MAX_PLACES+MAX);
				sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
				Command = Command + strlen(Command);
				for(i=1; i<ARGsCount; i++)
				{
					memset(Buffer, 0x00, 17);
					if( (ARGs[i].IsRequired == 1) && (ARGs[i].AttackCount > 0) ) /*Required attack*/
					{	
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						n=l % ARGs[i].AttackCount;
						sprintf(Buffer, "%d:%d:%c%c", ARGs[i].AttackType[n], x, BFchar, 0x00);
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if(ARGs[i].IsStatic)
					{
						sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
						Command = Command + strlen(ARGs[i].Arg) + 1;
					}
				}
				*(Command-1) = 0x00;
				printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
				system(CommandBeg);
				Command=CommandBeg;
				userFunc();
			}
		  }
		}
		if (OneNonRequiredField)
		{
		  m = RMaxAttackCount;
		  if ( rMaxAttackCount > m)
			m = rMaxAttackCount;
		  for(l=0; l<m; l++)
		  {
			for(x=BFstart; x<=BFstop; x=x+BFinc)
			{
				memset(Command, 0x00, ARGsCount*MAX_PLACES+MAX);
				sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
				Command = Command + strlen(Command);
				for(i=1; i<ARGsCount; i++)
				{
					memset(Buffer, 0x00, 17);
					if( (ARGs[i].IsRequired == 1) && (ARGs[i].AttackCount > 0) ) /*Required attack*/
					{
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						n=l % ARGs[i].AttackCount;
						sprintf(Buffer, "%d:%d:%c%c", ARGs[i].AttackType[n], x, BFchar, 0x00);
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 0) && (ARGs[i].IsStatic == 0) )
					{
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						n=l % ARGs[i].AttackCount;
						sprintf(Buffer, "%d:%d:%c%c", ARGs[i].AttackType[n], x, BFchar, 0x00);
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if(ARGs[i].IsStatic)
					{
						sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
						Command = Command + strlen(ARGs[i].Arg) + 1;
					}
				}
				*(Command-1) = 0x00;
				printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
				system(CommandBeg);
				Command=CommandBeg;
				userFunc();
			}
		  }
		}	
		if(OneCycleField) 
		{
		  m = 1;
		  if ( rMaxAttackCount > m)
			m = rMaxAttackCount;
		  if ( RMaxAttackCount > m)
			m = RMaxAttackCount;
		  for(l=0; l<m; l++)
		  {
			j=0;
			k=0;
			for(x=BFstart; x<=BFstop; x=x+BFinc)
			{
				memset(Command, 0x00, (ARGsCount*MAX_PLACES+MAX));
				sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
				Command = Command + strlen(Command);
				for(i=1; i<ARGsCount; i++)
				{
					memset(Buffer, 0x00, 17);
					if( (ARGs[i].IsRequired == 1) && (ARGs[i].AttackCount > 0) ) /*Required attack*/
					{
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						n=l % ARGs[i].AttackCount;
						sprintf(Buffer, "%d:%d:%c%c", ARGs[i].AttackType[n], x, BFchar, 0x00);
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 0) && (ARGs[i].IsStatic == 0) )
					{
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						n=l % ARGs[i].AttackCount;
						sprintf(Buffer, "%d:%d:%c%c", ARGs[i].AttackType[n], x, BFchar, 0x00);
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 1) && (ARGs[i].IsStatic == 0) )
					{
						sprintf(Buffer, "%s%c%c", ARGs[i].Arg, ARGs[i].CycleData[j], 0x00); 
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
						j=(j+1) % (ARGs[i].CycleCount);
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 2) && (ARGs[i].IsStatic == 0) )
					{
						sprintf(Buffer, "%s%c%c", ARGs[i].Arg, ARGs[i].CycleData[k], 0x00); 
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
						k=(k+1) % (ARGs[i].CycleCount);
					}
					else if(ARGs[i].IsStatic)
					{
						sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
						Command = Command + strlen(ARGs[i].Arg) + 1;
					}
				}
				*(Command-1) = 0x00;
				printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
				system(CommandBeg);
				Command=CommandBeg;
				userFunc();
			}
		   }
		}
		if( (OneRequiredField==0) && (OneNonRequiredField==0) && (OneCycleField==0) ) /*static string*/
		{
			memset(Command, 0x00, ARGsCount*MAX_PLACES+MAX);
			sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
			Command = Command + strlen(Command);
			for(i=1; i<ARGsCount; i++)
			{
				sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
				Command = Command + strlen(ARGs[i].Arg) +1;
			} 
			*(Command-1) = 0x00;
			printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
			system(CommandBeg);
			Command=CommandBeg;
			userFunc();
		}
		free(Buffer);
		free(CommandBeg);
		memset(line, 0x00, MAX);
		/*Ends the code that construced the ultimate commands for fuzzing*/
	}
       }
	break;
    }
    
    case 'n':
    {
	if ( argc == 6)
	{
		if( (fp = fopen(argv[2], "r")) == NULL)
		{
			printf("\nError opening file!!\n\n");
			exit(-1);
		}
		if( (atoi(argv[3]) || atoi(argv[4])) > 4294967294)
		{
			printf("\nNumber too large!!\n");
			exit(-1);
		}
		else
		{
			NUMstart=atoi(argv[3]);
			NUMstop=atoi(argv[4]);
			NUMinc=atoi(argv[5]);
			DefaultNumAttack=0;
			Loop=1;
		}
	}
	else if (argc != 3)
	{	
		Usage();
		exit(-1);
	}
	/*Read in the file and process it one line at a time*/
	memset(line, 0x00, MAX);
	LineNum=0;
	while (fgets (line, MAX, fp) != NULL)
	{
	  if(*line !=  '#')
	  {
		for(i=0; i<MAX/2; i++)
		{
			ARGs[i].Arg = NULL;		
		}	
		i=0;
		LineNum++;
		ARGsCount=0;
		Binary = strchr(line, 0x0a);
		*(Binary)=0x00;
		ARGs[i].Arg=line;
		printf("\n\tCmdline is parsing the following line: '%s'\n\n",ARGs[i].Arg);
		while ( (Binary = strchr(ARGs[i].Arg, 0x20)) != NULL )
		{
			*(Binary) = 0x00;
			i++;
			Binary++;
			ARGs[i].Arg=Binary;
			ARGsCount++;
		}
		ARGsCount++;
		for(i=0; i<ARGsCount; i++)
		{
			ARGs[i].IsRequired=0;
			ARGs[i].IsStatic=0;
			ARGs[i].AttackCount=0;
			ARGs[i].CycleType=0;
			ARGs[i].CycleCount=0;
		}
	
		/*This is the master loop that parses the ARGs on each line and builds the 'Command' buffer to call system on*/
		OneRequiredField = 0;
		OneNonRequiredField = 0;
		OneCycleField = 0;
		for(i=1; i<ARGsCount; i++)
		{
			/*First we look for the '_'*/
			Current=strchr(ARGs[i].Arg, '_');
			if (Current != NULL)
			{
				switch(*(Current+1))
				{
					case 'R':
						OneRequiredField = 1;
						ARGs[i].IsRequired = 1;
						break;

					case 'r':
						OneNonRequiredField = 1;
						ARGs[i].IsRequired = 0;	
						break;
				}
			}
			if ( Current=strchr(ARGs[i].Arg, '*') )
			{
				OneCycleField = 1;
				ARGs[i].CycleType = 1;
				ARGs[i].CycleCount = 255;
				for (j=0; j<ARGs[i].CycleCount; j++)
				{
					ARGs[i].CycleData[j] = j+1;
				}
				*(Current) = 0x00;
			}
			else if ( Current=strchr(ARGs[i].Arg, '[') )
			{	
				OneCycleField = 1;
				ARGs[i].CycleType = 2;
				Working=strchr(ARGs[i].Arg, ']');
				ARGs[i].CycleCount = (Working - Current) - 1;
				for(j=0; j<ARGs[i].CycleCount; j++)
				{
					ARGs[i].CycleData[j] = *(Current + j + 1);
				}
				*(Current) = 0x00;
			}				
			else if ( (strchr(ARGs[i].Arg, '_')) == NULL) 
			{
				ARGs[i].IsStatic=1;
			}
			 /*Ends first PARSING loop through the fields on one line*/
		}
		
		if ( (Command = malloc(ARGsCount*MAX_PLACES+MAX) ) == NULL)
		{
			printf("\n Memory error!!  Are there too many ARGs or too large a BUFFERMAX??\n\n");
			exit(-1);
		}
		CommandBeg=Command;
		Buffer=malloc(12);
		/*Array of field structures is populated. Begin the fuzzing*/
		if(OneRequiredField)
		{
		  m=0;
		  for(l=0; l<Loop; l++)
		  {
			if ( m == 5)	
			{
				NUMstart=0;
				NUMstop=23;
				NUMinc=1;
			}
			for(x=NUMstart; x<=NUMstop; x=x+NUMinc)
			{	
				memset(Command, 0x00, ARGsCount*MAX_PLACES+MAX);
				sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
				Command = Command + strlen(Command);
				for(i=1; i<ARGsCount; i++)
				{	
					memset(Buffer, 0x00, 12);
					if(ARGs[i].IsRequired == 1) /*Required attack*/
					{	
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						if( (m == 5) && (DefaultNumAttack == 1) )
							sprintf(Buffer, "%s ", NUMarray[x]);
						else
							sprintf(Buffer, "%d%c", x, 0x00);

						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if(ARGs[i].IsStatic)
					{
						sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
						Command = Command + strlen(ARGs[i].Arg) + 1;
					}
				}
				*(Command-1) = 0x00;
				printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
				system(CommandBeg);  /*You're going down!*/
				Command=CommandBeg;
				userFunc();
			}
		  m++;
		  NUMstart = NUMstart * 10;
		  NUMstop = NUMstop * 10;
		  NUMinc = NUMinc * 10;
		  }
		}
		
		if(DefaultNumAttack==0)
		{
			NUMstart=atoi(argv[3]);
			NUMstop=atoi(argv[4]);
			NUMinc=atoi(argv[5]);
		}
		else
		{
			NUMstart=1;
			NUMstop=10;
			NUMinc=1;
		}
		if(OneNonRequiredField)
		{
		  m=0;
		  for(l=0; l<Loop; l++)
		  {
			if ( m == 5)	
			{
				NUMstart=0;
				NUMstop=23;
				NUMinc=1;
			}
			for(x=NUMstart; x<=NUMstop; x=x+NUMinc)
			{	
				memset(Command, 0x00, ARGsCount*MAX_PLACES+MAX);
				sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
				Command = Command + strlen(Command);
				for(i=1; i<ARGsCount; i++)
				{	
					memset(Buffer, 0x00, 12);
					if(ARGs[i].IsRequired == 1) /*Required attack*/
					{	
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						if( (m == 5) &&  (DefaultNumAttack == 1) )
							sprintf(Buffer, "%s ", NUMarray[x]);
						else
							sprintf(Buffer, "%d%c", x, 0x00);

						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 0) && (ARGs[i].IsStatic == 0) )
					{
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						if( (m == 5) && (DefaultNumAttack == 1) )
							sprintf(Buffer, "%s ", NUMarray[x]);
						else
							sprintf(Buffer, "%d%c", x, 0x00);

						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if(ARGs[i].IsStatic)
					{
						sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
						Command = Command + strlen(ARGs[i].Arg) + 1;
					}
				}
				*(Command-1) = 0x00;
				printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
				system(CommandBeg);
				Command=CommandBeg;
				userFunc();
			}
		  m++;
		  NUMstart = NUMstart * 10;
		  NUMstop = NUMstop * 10;
		  NUMinc = NUMinc * 10;
		  }
		}

		if(DefaultNumAttack==0)
		{
			NUMstart=atoi(argv[3]);
			NUMstop=atoi(argv[4]);
			NUMinc=atoi(argv[5]);
		}
		else
		{
			NUMstart=1;
			NUMstop=10;
			NUMinc=1;
		}
		if(OneCycleField)
		{
		  m=0;
		  for(l=0; l<Loop; l++)
		  {
			if ( m == 5)	
			{
				NUMstart=0;
				NUMstop=23;
				NUMinc=1;
			}
			j=0;
			k=0;
			for(x=NUMstart; x<=NUMstop; x=x+NUMinc)
			{	
				memset(Command, 0x00, ARGsCount*MAX_PLACES+MAX);
				sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
				Command = Command + strlen(Command);
				for(i=1; i<ARGsCount; i++)
				{	
					memset(Buffer, 0x00, 12);
					if(ARGs[i].IsRequired == 1) /*Required attack*/
					{	
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						if( (m==5) && (DefaultNumAttack == 1) )
							sprintf(Buffer, "%s ", NUMarray[x]);
						else
							sprintf(Buffer, "%d%c", x, 0x00);

						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 0) && (ARGs[i].IsStatic == 0) )
					{
						Current=strchr(ARGs[i].Arg, '_');
						if( (y = Current - ARGs[i].Arg) != 0)
						{
							memcpy(Buffer, ARGs[i].Arg, y);
							sprintf(Command, "%s%c", Buffer, 0x20);
							Command = Command + strlen(Buffer) + 1;
							memset(Buffer, 0x00, 17);
						}
						if( (m == 5) && (DefaultNumAttack == 1) )
							sprintf(Buffer, "%s ", NUMarray[x]);
						else
							sprintf(Buffer, "%d%c", x, 0x00);

						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 1) && (ARGs[i].IsStatic == 0) )
					{
						sprintf(Buffer, "%s%c%c", ARGs[i].Arg, ARGs[i].CycleData[j], 0x00); 
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
						j=(j+1) % (ARGs[i].CycleCount);
					}
					else if( (ARGs[i].IsRequired == 0) && (ARGs[i].CycleType == 2) && (ARGs[i].IsStatic == 0) )
					{
						sprintf(Buffer, "%s%c%c", ARGs[i].Arg, ARGs[i].CycleData[k], 0x00); 
						sprintf(Command, "%s%c", Buffer, 0x20);
						Command = Command + strlen(Buffer) + 1;
						k=(k+1) % (ARGs[i].CycleCount);
					}
					else if(ARGs[i].IsStatic)
					{
						sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
						Command = Command + strlen(ARGs[i].Arg) + 1;
					}
				}
				*(Command-1) = 0x00;
				printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
				system(CommandBeg);
				Command=CommandBeg;
				userFunc();
			}
		  m++;
		  NUMstart = NUMstart * 10;
		  NUMstop = NUMstop * 10;
		  NUMinc = NUMinc * 10;
		  }
		}
		if( (OneRequiredField==0) && (OneNonRequiredField==0) && (OneCycleField==0) ) /*static string*/
		{
			memset(Command, 0x00, ARGsCount*MAX_PLACES+MAX);
			sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
			Command = Command + strlen(Command);
			for(i=1; i<ARGsCount; i++)
			{
				sprintf(Command, "%s%c", ARGs[i].Arg, 0x20);
				Command = Command + strlen(ARGs[i].Arg) +1;
			} 
			*(Command-1) = 0x00;
			printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
			system(CommandBeg);
			Command=CommandBeg;
			userFunc();
		}
	//printf("CommandBeg=%x\n", &CommandBeg);
	//fflush(stdout);
	//	free(CommandBeg);  Was causing coredumps for some reason
		free(Buffer);
		memset(line, 0x00, MAX);
		/*Ends the code that construced the ultimate commands for fuzzing*/
	}
       }	
	break;
    }

    case 'r':
    {
	if ( argc == 4)
	{
		if( (fp = fopen(argv[2], "r")) == NULL)
		{
			printf("\nError opening file!!\n\n");
			exit(-1);
		}
		if( (atoi(argv[3]) || atoi(argv[4])) > BUFFERMAX)
		{
			printf("\nBuffer size to large!!\n");
			exit(-1);
		}
		else
		{
			Seed=atoi(argv[3]);
			srand(Seed);
			srand48(Seed);
		}
	}
	else if (argc != 3)
	{	
		Usage();
		exit(-1);
	}
	/*Read in the file and process it one line at a time*/
	memset(line, 0x00, MAX);
	LineNum=0;
	while (fgets (line, MAX, fp) != NULL)
	{
	  if(*line !=  '#')
	  {
		for(i=0; i<MAX/2; i++)
		{
			ARGs[i].Arg = NULL;		
		}	
		i=0;
		LineNum++;
		ARGsCount=0;
		Binary = strchr(line, 0x0a);
		*(Binary)=0x00;
		ARGs[i].Arg=line;
		printf("\n\tCmdline is parsing the following line: '%s'\n\n",ARGs[i].Arg);
		while ( (Binary = strchr(ARGs[i].Arg, 0x20)) != NULL )
		{
			*(Binary) = 0x00;
			i++;
			Binary++;
			ARGs[i].Arg=Binary;
			ARGsCount++;
		}
		ARGsCount++;

		if ( (Command = malloc(255*MAX_PLACES+MAX)) == NULL) 
		{
			printf("\n Memory error!!  Are there to many ARGs or too large a BUFFERMAX??\n\n");
			exit(-1);
		}
		CommandBeg=Command;
		Buffer=malloc(MAX_PLACES);
		/*Now that the array of field structures is populated, we'll begin the fuzzing*/
		o = ( rand() & 0xf );
		for(x=0; x<=o; x++)
		{
			memset(CommandBeg, 0x00, 255*MAX_PLACES+MAX);
			sprintf(Command, "./cmd %s%c", ARGs[0].Arg, 0x20);
			Command = Command + strlen(Command);
			if( (m=rand()&0xff) > 150)
				p=rand()&0xff;
			else if( m < ARGsCount )
				p=m;
			else
				p=ARGsCount; 
			for(i=1; i<p; i++)
			{
				memset(Buffer, 0x00, MAX_PLACES);
				if( (m=rand()&0xff) > 100)
				{
					if( (m=rand()&0xff) > 127)
						n=1;
					else
						n=2;
					q=rand()&0xff;
					if( q > 64 && q < 123  )
					{
						sprintf(Buffer, "-%c%c%d:%d:%c%c", q, 0x20, n, (rand()&0xffff), (rand()&0xff), 0x00);
					}
					else
					{
						sprintf(Buffer, "%d:%d:%c%c", n, (rand()&0xffff), (rand()&0xff), 0x00);
					}	
				}
				else if (m > 60)
				{
					n= rand()&0xffffff;
					q=rand()&0xff;
					if( q > 64 && q < 123  )
					{
						sprintf(Buffer, "-%c%c%d%c", q, 0x20, n, 0x00);
					}
					else
					{
						sprintf(Buffer, "%d%c", n, 0x00);
					}	
				}
				else if (m > 20)
				{
					a = drand48();
					a = a*100000;
					q=rand()&0xff;
					if( q > 64 && q < 123  )
					{
						sprintf(Buffer, "-%c%c%f%c", q, 0x20, a, 0x00);
					}
					else
					{
						sprintf(Buffer, "%f%c", a, 0x00);
					}	
				}
				else if (m > 10)
				{
					n= rand()&0xffffff;
					q=rand()&0xff;
					if( q > 64 && q < 123  )
					{
						sprintf(Buffer, "-%c%c%c%d%c", q, 0x20, '-', n, 0x00);
					}
					else
					{
						sprintf(Buffer, "%c%d%c", '-', n, 0x00);
					}	
				}
				else
				{
					a = drand48();
					a = a*100000;
					q=rand()&0xff;
					if( q > 64 && q < 123  )
					{
						sprintf(Buffer, "-%c%c%c%f%c", q, 0x20, '-', a, 0x00);
					}
					else
					{
						sprintf(Buffer, "%c%f%c", '-', a, 0x00);
					}	
				}
				sprintf(Command, "%s%c", Buffer, 0x20);
				Command = Command + strlen(Buffer) + 1;
			}
			*(Command-1) = 0x00;
			printf("\t Cmdline constucted and is executing the following line: '%s'\n", CommandBeg);
			system(CommandBeg);
			Command=CommandBeg;
			userFunc();
		}
		free(CommandBeg);
		free(Buffer);
	}
       }
	break;
    }
    default:
	Usage();
	break;
  }

	fclose(fp);
	exit(0);
}


/*This is the routine to print a buffer in ASCII and hex*/
void PRINT(int count, const char *lable, const char *buffer)
{
	int i;
/*	printf("%s[chars]= ", lable);
	printf("\n"); */
	for(i=0; i<count; i++)
	{
		printf("%c", buffer[i]);
	}

	printf("\n");
/*	printf("%s[hex]= ", lable);
	printf("\n");*/
	for(i=0; i<count; i++)
	{
		printf("%x ", (0xff&buffer[i]));
	}
	printf("\n");
}

void Usage()
{
	printf("\nBy: Jared DeMott - Applied Security, Inc.\n\n");
	printf("Cmdline is used to fuzz local *NIX applications. It parses a config file containing\n");
	printf("program names and args/identifiers on each line, and iteratively calls 'cmd'.\n");
	printf("See testfile for an example config file.\n\n");
	printf("Usage:\n");
	printf("  ./cmdline -b filename [start] [stop] [inc] [char]\n");
	printf("  ./cmdline -n filename [start] [stop] [inc]\n");
	printf("  ./cmdline -r filename [seed]\n\n");

	printf("      -b - Buffer/Format attack\n");
	printf("      -n - Number attack\n");
	printf("      -r - random attack\n");
	printf("filename - config file detailing how to stress application parameters\n");
	printf("           Each line in the file should contain a command and parameter\n");
	printf("           identifiers.\n");
	printf("   start - initial number of characters to use\n");
	printf("    stop - max number of characters to use\n");
	printf("     inc - amount of characters to increment by\n");
	printf("    char - ASCII hex value of the character to use with no leading 0x (for example, 41 for 'A')\n");
	printf("    seed - value to seed random number generator\n\n");

	printf("Argument syntax in config files:\n");
	printf("      _r1 - optional argument buffer attack (such as -s for ping)\n");
	printf("      _r2 - optional argument format attack\n");
	printf("      _R1 - required argument buffer attack (such as hostname for ping)\n");
	printf("      _R2 - required argument format attack\n");
	printf("    [xyz] - x, y, z will be options used in a cyclic manner for that position\n");
	printf("        * - cycle through all letters (upper and lower) and numbers \n\n");
	printf("Processing/Fuzzing: \n");
	printf(" - Program name always in command\n");
	printf(" - options with no identifiers (static strings) are always in command\n");
	printf(" - Required fields are always in command \n");
	printf(" - First, fuzz required fields \n");
	printf(" - Second, add non-required feilds \n");
	printf(" - Third, begin fuzzing all fields and cycling through all * or [] indicators\n\n");

	printf("Buffer/Format Attack (-b) :\n");
	printf("  Example config line:  'ping -i _R1 _r1 -[xyz] -*' \n");
	printf("  Example command line: './cmdline -b testfile 100 200 100 A'\n\n");
	printf("  This will call:\n");
	printf("    1. 'ping -i <100 A's>'\n");
	printf("    2. 'ping -i <200 A's>'\n");
	printf("    3. 'ping -i <100 A's> <100 A's>\n");
	printf("    4. 'ping -i <200 A's> <200 A's>\n");
	printf("    5. 'ping -i <100 A's> <100 A's> -x -0x01\n");
	printf("    6. 'ping -i <200 A's> <200 A's> -y -0x02\n\n");

	printf("Number Attack (-n):\n");
	printf("   Example config line:  'ping -K _R1 _r1 -[abc]' \n");
	printf("   Example command line: './cmdline -n testfile 1 10 1'\n\n");
	printf("    1. 'ping -K 1'\n");
	printf("    2. 'ping -K 2'\n");
	printf("        ...\n");
	printf("    11. 'ping -K 1 1\n");
	printf("    12. 'ping -K 2 1\n");
	printf("        ...\n");
	printf("    21. 'ping -K 1 1 -a\n");
	printf("    22. 'ping -K 2 2 -b\n");
	printf("    23. 'ping -K 3 3 -c\n");
	printf("    24. 'ping -K 4 4 -a\n");
	printf("        ...\n\n");
	printf("Also, in default mode the number attack performs a logarithmic increment followed by a boundry attack.\n\n");

	printf("Random Attack (-r):\n");
	printf("   The program names from the config file are used, followed by a random number of arguments with\n");
	printf("   random contents, sizes, attack types, and/or static characters.  The arguments may contain\n");
	printf("   floating point numbers, and who knows what else with all that randomness!\n\n");
	
	printf("Customization:\n");
	printf("   The function in userFunc.h is called with each test. This may be customized\n");
	printf("   to perform additional actions including killing an existing process.\n");
}

